/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.process.internal.definitions.ide.ui.deliver.editors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IInputValidator;

import com.ibm.team.process.internal.ide.ui.editors.form.IDevelopmentLineProxy;

/**
 * Dialog used by the CreateTeamAreasAspectEditor when creating a root team
 * area.
 */
public class CreateRootTeamAreaDialog extends Dialog {
	
	private Text fName;
	private Combo fCombo;
	private final IDevelopmentLineProxy[] fLineProxies;
	
	private String fNameResult;
	private String fLineResult;
	private Composite fComposite;
	private Label fErrorLabel;
	private final IInputValidator fValidator;

	protected CreateRootTeamAreaDialog(Shell parentShell, IDevelopmentLineProxy[] lineProxies, IInputValidator validator) {
		super(parentShell);
		setShellStyle(getShellStyle() | SWT.RESIZE);
		fLineProxies = lineProxies;
		fValidator = validator;
	}
	
	/*
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(Messages.CreateRootTeamAreaDialog_0);
	}

	/**
	 * Returns the name that the user input into the dialog. If called before
	 * the user presses the OK button, this method will return
	 * <code>null</code>.
	 * 
	 * @return the name input by the user or <code>null</code> if the user has
	 *         not pressed the OK button
	 */
	public String getName() {
		return fNameResult;
	}

	/**
	 * Returns the id of the development line that the user selected in the
	 * dialog. Returns <code>null</code> if the user did not select a timeline
	 * or if this method is called before the user presses the OK button.
	 * 
	 * @return the id of the development line selected by the user or
	 *         <code>null</code>
	 */
	public String getDevelopmentLine() {
		return fLineResult;
	}
	
	/*
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
		fComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginTop = 10;
		fComposite.setLayout(layout);
		fComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
		applyDialogFont(fComposite);
		
		// Wrap the description label in a composite to add a little margin space at the bottom.
		Composite descriptionComposite = new Composite(fComposite, SWT.NONE);
		descriptionComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		layout = new GridLayout();
		layout.marginWidth = 0;
		layout.marginHeight = 0;
		layout.marginBottom = 10;
		descriptionComposite.setLayout(layout);
		
		Label description = new Label(descriptionComposite, SWT.WRAP);
		description.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		description.setText(Messages.CreateRootTeamAreaDialog_1);
		
		Label nameLabel = new Label(fComposite, SWT.WRAP);
		nameLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		nameLabel.setText(Messages.CreateRootTeamAreaDialog_2);
		
		fName = new Text(fComposite, SWT.SINGLE | SWT.BORDER);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.minimumWidth = 500;
		fName.setLayoutData(gridData);
		fName.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validateInput();
			}
		});
		
		Label label = new Label(fComposite, SWT.WRAP);
		label.setText(Messages.CreateRootTeamAreaDialog_3);
		label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		fCombo = new Combo(fComposite, SWT.DROP_DOWN | SWT.READ_ONLY);
		fCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		String none = Messages.CreateRootTeamAreaDialog_4;
		fCombo.add(none);
		fCombo.setText(none);
		for (int i = 0; i < fLineProxies.length; i++) {
			fCombo.add(fLineProxies[i].getName());
		}
		
		// Wrap the error label in a composite to add a little margin space at the top.
		Composite labelComposite = new Composite(fComposite, SWT.NONE);
		labelComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		layout = new GridLayout();
		layout.marginWidth = 0;
		layout.marginTop = layout.marginHeight;
		layout.marginHeight = 0;
		labelComposite.setLayout(layout);
		
		fErrorLabel = new Label(labelComposite, SWT.WRAP);
		fErrorLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		fErrorLabel.setText(""); //$NON-NLS-1$
		
		return fComposite;
	}
	
	/*
	 * @see org.eclipse.jface.dialogs.Dialog#create()
	 */
	public void create() {
		super.create();
		validateInput();
	}
	
	protected void validateInput() {
		if (fValidator != null) {
			String text = fName.getText();
			String errorText = fValidator.isValid(text);
			getButton(OK).setEnabled(errorText == null);
			fErrorLabel.setText(errorText == null ? "" : errorText); //$NON-NLS-1$
			updateShellSize();
		}
	}

	/**
	 * If necessary, makes the shell taller so all the content can fit.
	 */
	private void updateShellSize() {
		Point beforeSize = fComposite.getSize();
		fComposite.layout();
		Point afterSize = fComposite.computeSize(beforeSize.x, SWT.DEFAULT, true);
		int yDiff = afterSize.y - beforeSize.y;
		if (yDiff > 0) {
			Point shellSize = getShell().getSize();
			getShell().setSize(new Point(shellSize.x, shellSize.y + yDiff));
		}
	}

	protected void buttonPressed(int buttonId) {
        if (buttonId == IDialogConstants.OK_ID) {
        	int index = fCombo.getSelectionIndex();
			if (index > 0 && index <= fLineProxies.length) { // Index 0 == "<None>"
				fLineResult = fLineProxies[index - 1].getId();
			}
			fNameResult = fName.getText();
        }
        super.buttonPressed(buttonId);
    }

}
