/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.process.internal.definitions.ide.ui.deliver.editors;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.osgi.util.NLS;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;

import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.jface.JazzResources;

import com.ibm.team.process.common.ModelElement;

import com.ibm.team.process.internal.definitions.ide.ui.DefinitionsIdeUIPlugin;

import com.ibm.team.process.ide.ui.IProcessAspectEditorSite;
import com.ibm.team.process.ide.ui.OperationDetailsAspectEditor;
import com.ibm.team.process.ide.ui.ProcessAspect;

/**
 * Aspect editor for the details of the "create invitation work items" participant.
 * Allows the user to specify some work item templates.
 */
public class CreateWorkItemsAspectEditor extends OperationDetailsAspectEditor {
	
	static final String TAG_WORKITEMS = "workitems"; //$NON-NLS-1$
	static final String TAG_WORKITEM = "workitem"; //$NON-NLS-1$
	static final String ATTR_TYPE = "type"; //$NON-NLS-1$
	static final String ATTR_CATEGORY_PATH = "categoryPath"; //$NON-NLS-1$
	static final String ATTR_SUMMARY = "summary"; //$NON-NLS-1$
	static final String ATTR_DESCRIPTION = "description"; //$NON-NLS-1$
	static final String ATTR_HTML_DESCRIPTION = "htmlDescription"; //$NON-NLS-1$
	
	static final String VARS_TOOLTIP = Messages.SetupProjectAspectEditor_insertVar;
	static final String[] VAR_PROJECT_AREA_NAME = {Messages.SetupProjectAspectEditor_projectAreaName, "${project-area-name}"}; //$NON-NLS-1$
	static final String[] VAR_TEAM_AREA_NAME = {Messages.SetupProjectAspectEditor_teamAreaName, "${team-area-name}"}; //$NON-NLS-1$
	static final String[] VAR_PATH_SEPARATOR = {Messages.CreateWorkItemsAspectEditor_pathSeparator, "/"}; //$NON-NLS-1$
	static final String[][] CATEGORY_PATH_VARS = { VAR_PROJECT_AREA_NAME, VAR_TEAM_AREA_NAME, VAR_PATH_SEPARATOR };

	static final String WORK_ITEM_TYPES_DATA_ID = "com.ibm.team.workitem.configuration.workItemTypes"; //$NON-NLS-1$
	static final String TAG_WORKITEM_TYPE = "type"; //$NON-NLS-1$
	static final String ATTR_WORKITEM_TYPE_ID = "id"; //$NON-NLS-1$
	static final String ATTR_WORKITEM_TYPE_NAME = "name"; //$NON-NLS-1$

	static final ImageDescriptor DELETE_IMAGE_DESCRIPTOR = DefinitionsIdeUIPlugin.getImageDescriptor("icons/copied/elcl16/rem_co.gif"); //$NON-NLS-1$
	
	static class WorkItemType {
		String typeId;
		String typeName;

		public String toString() {
			if (typeName != null) {
				return typeName;
			}
			return typeId;
		}
	}
	
	/**
	 * Model object for a work item template.
	 */
	class WorkItemTemplate {
		WorkItemType type;
		String categoryPath;
		String summary;
		String description;
		boolean isHTML;

		public void readFrom(IMemento memento) {
			String typeId = getAttrib(memento, ATTR_TYPE, ""); //$NON-NLS-1$
			for (Iterator iterator = fWorkItemTypes.iterator(); iterator.hasNext();) {
				WorkItemType type = (WorkItemType) iterator.next();
				if (typeId.equals(type.typeId)) {
					this.type = type;
					break;
				}
			}
			if (this.type == null) {
				// This handles the case where the workitem type no longer exists.
				// In this case, we will still show the initial value as the current value in the XML.
				WorkItemType type = new WorkItemType();
				type.typeId = typeId;
				type.typeName = typeId;
				this.type = type;
			}
			categoryPath = getAttrib(memento, ATTR_CATEGORY_PATH, ""); //$NON-NLS-1$
			summary = getAttrib(memento, ATTR_SUMMARY, ""); //$NON-NLS-1$
			description = memento.getString(ATTR_DESCRIPTION);
			if (description != null) {
				isHTML = false;
			}
			else {
				description = memento.getString(ATTR_HTML_DESCRIPTION);
				if (description != null) {
					isHTML = true;
				}
				else {
					description = ""; //$NON-NLS-1$
					isHTML = false;
				}
			}
		}

		/**
		 * Saves the state of the template into a child memento of the given parent memento.
		 */
		public void createMemento(IMemento parentMemento) {
			IMemento memento = parentMemento.createChild(TAG_WORKITEM);
			if (type != null && type.typeId.length() > 0) {
				memento.putString(ATTR_TYPE, type.typeId);
			}
			if (categoryPath != null && categoryPath.length() > 0) {
				memento.putString(ATTR_CATEGORY_PATH, categoryPath);
			}
			memento.putString(ATTR_SUMMARY, summary);
			memento.putString(isHTML ? ATTR_HTML_DESCRIPTION : ATTR_DESCRIPTION, convertLineSeparators(description));
		}
	}

	/**
	 * Form UI for a work item template.
	 */
	class WorkItemTemplateForm {

		private WorkItemTemplate fTemplate;
		private Composite fComposite;
		private Text fCategoryPathText;
		private Label fSummaryLabel;
		private Text fSummaryText;
		private Text fDescriptionText;

		WorkItemTemplateForm(final WorkItemTemplate template, Composite parent, FormToolkit toolkit, int index) {
			fTemplate = template;
			fComposite = toolkit.createComposite(parent);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(fComposite);
			GridLayoutFactory.fillDefaults().applyTo(fComposite);
			
			if (index != 0) {
				Label separator = toolkit.createLabel(fComposite, "", SWT.SEPARATOR | SWT.HORIZONTAL); //$NON-NLS-1$
				GridDataFactory.fillDefaults().grab(true, false).applyTo(separator);
			}

			Composite summaryComposite = toolkit.createComposite(fComposite);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(summaryComposite);
			GridLayoutFactory.fillDefaults().numColumns(3).applyTo(summaryComposite);
			
			fSummaryLabel = toolkit.createLabel(summaryComposite, ""); //$NON-NLS-1$
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(fSummaryLabel);
			
			fSummaryText = toolkit.createText(summaryComposite, template.summary, SWT.SINGLE | SWT.BORDER | SWT.SEARCH);
			fSummaryText.setMessage(Messages.CreateWorkItemsAspectEditor_summary_message);
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).grab(true, false).hint(150, -1).applyTo(fSummaryText);
			
			ToolBar toolbar = new ToolBar(summaryComposite, SWT.NONE);
			toolkit.adapt(toolbar, false, false);
			ToolItem deleteItem = new ToolItem(toolbar, SWT.NONE);
			deleteItem.setImage(JazzResources.getImageWithDefault(fResourceManager, DELETE_IMAGE_DESCRIPTOR));
			deleteItem.setToolTipText(Messages.AcceptTeamInvitationAspectEditor_0);
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(toolbar);
			deleteItem.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					doDeleteTemplate(template, WorkItemTemplateForm.this);
				}
			});
			
			Composite typeComposite = toolkit.createComposite(fComposite);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(typeComposite);
			GridLayoutFactory.fillDefaults().numColumns(5).applyTo(typeComposite);
			
			Label typeLabel = toolkit.createLabel(typeComposite, Messages.CreateWorkItemsAspectEditor_0);
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(typeLabel);

			final ComboViewer typeViewer = new ComboViewer(typeComposite, SWT.BORDER);
			toolkit.adapt(typeViewer.getControl(), true, false);
			typeViewer.setContentProvider(new ArrayContentProvider());
			typeViewer.setLabelProvider(new LabelProvider());
			typeViewer.setInput(fWorkItemTypes);
			if (fTemplate.type != null) {
				typeViewer.getCombo().setText(fTemplate.type.toString());
			}
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).grab(true, false).hint(75, -1).applyTo(typeViewer.getControl());

			Label categoryPathLabel = toolkit.createLabel(typeComposite, Messages.CreateWorkItemsAspectEditor_1);
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(categoryPathLabel);

			fCategoryPathText = toolkit.createText(typeComposite, template.categoryPath, SWT.SINGLE | SWT.BORDER);
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).grab(true, false).hint(150, -1).applyTo(fCategoryPathText);

			createVarsButton(typeComposite, fCategoryPathText, toolkit, CATEGORY_PATH_VARS);
			
			Composite descriptionComposite = toolkit.createComposite(fComposite);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(descriptionComposite);
			GridLayoutFactory.fillDefaults().numColumns(3).applyTo(descriptionComposite);
			
			Label descriptionLabel = toolkit.createLabel(descriptionComposite, Messages.AcceptTeamInvitationAspectEditor_1);
			GridDataFactory.fillDefaults().grab(true, false).align(SWT.FILL, SWT.CENTER).applyTo(descriptionLabel);
			
			createRadioButton(descriptionComposite, toolkit, Messages.AcceptTeamInvitationAspectEditor_2, false);
			createRadioButton(descriptionComposite, toolkit, Messages.AcceptTeamInvitationAspectEditor_3, true);
			
			fDescriptionText = toolkit.createText(descriptionComposite, template.description, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL | SWT.BORDER);
			int h = fDescriptionText.getLineHeight() * 8;
			h = fDescriptionText.computeTrim(0, 0, 100, h).height;
			GridDataFactory.fillDefaults().grab(true, false).span(3, 1).hint(150, h).applyTo(fDescriptionText);
			
			ModifyListener modifyListener = new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					if (e.widget == fSummaryText) {
						fTemplate.summary = fSummaryText.getText();
						setDirty();
					}
					else if (e.widget == typeViewer.getCombo()) {
						IStructuredSelection selection = (IStructuredSelection) typeViewer.getSelection();
						fTemplate.type = (WorkItemType) selection.getFirstElement();
						setDirty();
					}
					else if (e.widget == fCategoryPathText) {
						fTemplate.categoryPath = fCategoryPathText.getText();
						setDirty();
					}
					else if (e.widget == fDescriptionText) {
						fTemplate.description = fDescriptionText.getText();
						setDirty();
					}
				}
			};
			typeViewer.getCombo().addModifyListener(modifyListener);
			fCategoryPathText.addModifyListener(modifyListener);
			fSummaryText.addModifyListener(modifyListener);
			fDescriptionText.addModifyListener(modifyListener);
			
			setIndex(index);
		}

		private void createRadioButton(Composite parent, FormToolkit toolkit, String text, final boolean isHTML) {
			Button button = toolkit.createButton(parent, text, SWT.RADIO);
			button.setSelection(fTemplate.isHTML == isHTML);
			button.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					if (((Button) e.widget).getSelection()) {
						fTemplate.isHTML = isHTML;
						setDirty();
					}
				}
			});
		}

		void setIndex(int index) {
			fSummaryLabel.setText(NLS.bind(Messages.AcceptTeamInvitationAspectEditor_4, Integer.toString(index + 1)));
		}
		
		void dispose() {
			fComposite.dispose();
		}

		public void activate() {
			// hack: give the description field focus first, to have the managed form 
			// scroll and reveal more of the work item form if needed; without this,
			// only the summary line is revealed.
			fDescriptionText.setFocus();
			fSummaryText.setFocus();
			fSummaryText.selectAll();
		}
		
	}
	
	private List fTemplates = new ArrayList();
	private List fTemplateForms = new ArrayList();
	private Label fCountLabel;
	protected FormToolkit fToolkit;
	protected LocalResourceManager fResourceManager;
	private Composite fContentArea;
	private List fWorkItemTypes;


	/**
	 * Converts line separators from the platform convention to "\n".
	 */
	static String convertLineSeparators(String text) {
		String lineSep = System.getProperty("line.separator"); //$NON-NLS-1$
		if (lineSep != null) {
			return text.replaceAll(lineSep, "\n"); //$NON-NLS-1$
		}
		return text;
	}

	static String getAttrib(IMemento memento, String attr, String defaultValue) {
		String val = null;
		if (memento != null) {
			val = memento.getString(attr);
		}
		return val != null ? val : defaultValue;
	}
	
	/**
	 * Returns the work item types defined in the configuration data.
	 */
	private List getWorkItemTypes() {
		List types = new ArrayList();
		ModelElement typesElement = getSite().getConfigurationData(WORK_ITEM_TYPES_DATA_ID);
		if (typesElement != null) {
			List children = typesElement.getChildElements();
			for (Iterator i = children.iterator(); i.hasNext();) {
				ModelElement element = (ModelElement) i.next();
				if (TAG_WORKITEM_TYPE.equals(element.getName())) {
					String id = element.getAttribute(ATTR_WORKITEM_TYPE_ID);
					if (id != null) {
						WorkItemType type = new WorkItemType();
						type.typeId = id;
						String name = element.getAttribute(ATTR_WORKITEM_TYPE_NAME);
						if (name != null) {
							type.typeName = name;
						}
						types.add(type);
					}
				}
			}
		}
		return types;
	}
	
	/*
	 * @see com.ibm.team.process.ide.ui.ProcessAspectEditor#init(com.ibm.team.process.ide.ui.IProcessAspectEditorSite, com.ibm.team.process.ide.ui.ProcessAspect)
	 */
	public void init(IProcessAspectEditorSite site, ProcessAspect inputAspect) {
		super.init(site, inputAspect);
		fWorkItemTypes = getWorkItemTypes();
	}

	public void restoreState(IMemento memento) {
		List templates = new ArrayList();
		IMemento workitemsMemento = memento.getChild(TAG_WORKITEMS);
		if (workitemsMemento != null) {
			IMemento[] children = workitemsMemento.getChildren(TAG_WORKITEM);
			for (int i = 0; i < children.length; i++) {
				WorkItemTemplate template = new WorkItemTemplate();
				template.readFrom(children[i]);
				templates.add(template);
			}
		}
		fTemplates = templates;
	}

	/**
	 * Container is asking to save the changes. Saves the templates into the given memento.
	 */
	public boolean saveState(IMemento memento) {
		IMemento workitemsMemento = memento.getChild(TAG_WORKITEMS);
		if (workitemsMemento == null) {
			workitemsMemento = memento.createChild(TAG_WORKITEMS);
		}
		for (Iterator i = fTemplates.iterator(); i.hasNext();) {
			WorkItemTemplate template = (WorkItemTemplate) i.next();
			template.createMemento(workitemsMemento);
		}
		return true;
	}

	public void createControl(Composite parent, FormToolkit toolkit) {
		fToolkit = toolkit;
		parent.setLayout(new FillLayout());
		
		Composite composite = toolkit.createComposite(parent);
		GridLayoutFactory.fillDefaults().applyTo(composite);
		
		fResourceManager = new LocalResourceManager(JFaceResources.getResources(), composite);
		
		createContents(composite, toolkit);
		
		updateUI();
	}

	protected void createContents(Composite parent, FormToolkit toolkit) {
		createTemplatesHeader(parent, toolkit);
		createContentArea(parent, toolkit);
	}

	protected void createTemplatesHeader(Composite parent, FormToolkit toolkit) {
		Composite composite = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(composite);
		GridLayoutFactory.fillDefaults().numColumns(2).applyTo(composite);
		
		fCountLabel = toolkit.createLabel(composite, ""); //$NON-NLS-1$
		fCountLabel.setFont(getBoldFont());
		GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).grab(true, true).applyTo(fCountLabel);
		
		Button newButton = toolkit.createButton(composite, Messages.AcceptTeamInvitationAspectEditor_5, SWT.PUSH);
		newButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				doNewTemplate();
			}
		});
	}

	protected Composite createContentArea(Composite parent, FormToolkit toolkit) {
		fContentArea = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(fContentArea);
		
		GridLayoutFactory.fillDefaults().spacing(0, 10).applyTo(fContentArea);
		
		return fContentArea;
	}

	protected static Font getBoldFont() {
		return JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);
	}


	protected void updateUI() {
		disposeForms();
		createForms();
		updateCount();
	}

	private void updateCount() {
		fCountLabel.setText(NLS.bind(Messages.AcceptTeamInvitationAspectEditor_6, Integer.toString(fTemplates.size())));
	}

	private void createForms() {
		int index = 0;
		for (Iterator i = fTemplates.iterator(); i.hasNext();) {
			WorkItemTemplate template = (WorkItemTemplate) i.next();
			fTemplateForms.add(new WorkItemTemplateForm(template, getContentArea(), fToolkit, index++));
		}
	}

	private void disposeForms() {
		for (Iterator i = fTemplateForms.iterator(); i.hasNext();) {
			WorkItemTemplateForm form = (WorkItemTemplateForm) i.next();
			form.dispose();
		}
		fTemplateForms.clear();
	}

	private void doNewTemplate() {
		WorkItemTemplate template = new WorkItemTemplate();
		template.summary = Messages.AcceptTeamInvitationAspectEditor_7;
		template.description = Messages.AcceptTeamInvitationAspectEditor_8;
		template.isHTML = false;
		
		WorkItemTemplateForm form = new WorkItemTemplateForm(template, getContentArea(), fToolkit, fTemplates.size());
		fTemplates.add(template);
		fTemplateForms.add(form);
		
		updateCount();
		setDirty();
		
		getContentArea().layout();
		getSite().reflow();

//		fScrolledForm.setOrigin(0, 65535); // scroll to bottom
		form.activate();
	}

	protected Composite getContentArea() {
		return fContentArea;
	}
	
	private void doDeleteTemplate(WorkItemTemplate template, WorkItemTemplateForm form) {
		if (MessageDialog.openQuestion(getShell(), Messages.AcceptTeamInvitationAspectEditor_9, NLS.bind(Messages.AcceptTeamInvitationAspectEditor_10, template.summary))) {
			form.dispose();
			fTemplates.remove(template);
			fTemplateForms.remove(form);
			
			updateCount();
			updateIndices();
			setDirty();
			
			getSite().reflow();
		}
	}

	protected Shell getShell() {
		return getContentArea().getShell();
	}

	private void updateIndices() {
		int index = 0;
		for (Iterator i = fTemplateForms.iterator(); i.hasNext();) {
			WorkItemTemplateForm form = (WorkItemTemplateForm) i.next();
			form.setIndex(index++);
		}
	}

	public void dispose() {
	}

	/**
	 * Creates the arrow button for inserting variables.
	 */
	protected VarsButton createVarsButton(Composite composite, Text text, FormToolkit toolkit, String[][] vars) {
		VarsButton varsButton = new VarsButton(composite, VARS_TOOLTIP, vars);
		toolkit.adapt(varsButton.getButton(), true, false);
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.BEGINNING).applyTo(varsButton.getButton());
		varsButton.setText(text);
		return varsButton;
	}

}
